@@ -6,22 +6,23 @@
 module Ide.Plugin.CabalFmt where
 
 import           Control.Lens
-import           Control.Monad.Except        (throwError)
+import           Control.Monad.Except             (throwError)
 import           Control.Monad.IO.Class
-import qualified Data.Text                   as T
-import           Development.IDE             hiding (pluginHandlers)
-import           Ide.Plugin.Error            (PluginError (PluginInternalError, PluginInvalidParams))
+import qualified Data.Text                        as T
+import           Development.IDE                  hiding (pluginHandlers)
+import           Development.IDE.Core.PluginUtils (mkFormattingHandlers)
+import           Ide.Plugin.Error                 (PluginError (PluginInternalError, PluginInvalidParams))
 import           Ide.Plugin.Properties
 import           Ide.PluginUtils
 import           Ide.Types
-import qualified Language.LSP.Protocol.Lens  as L
+import qualified Language.LSP.Protocol.Lens       as L
 import           Language.LSP.Protocol.Types
-import           Prelude                     hiding (log)
+import           Prelude                          hiding (log)
 import           System.Directory
 import           System.Exit
 import           System.FilePath
 import           System.Process.ListLike
-import qualified System.Process.Text         as Process
+import qualified System.Process.Text              as Process
 
 data Log
   = LogProcessInvocationFailure Int
@@ -64,7 +65,7 @@ provider recorder _ _ _ (FormatRange _) _ _ _ = do
   throwError $ PluginInvalidParams "You cannot format a text-range using cabal-fmt."
 provider recorder plId ideState _ FormatText contents nfp opts = do
   let cabalFmtArgs = [ "--indent", show tabularSize]
-  cabalFmtExePath <- fmap T.unpack $ liftIO $ runAction "cabal-gild" ideState $ usePropertyAction #path plId properties
+  cabalFmtExePath <- fmap T.unpack $ liftIO $ runAction "cabal-fmt" ideState $ usePropertyAction #path plId properties
   x <- liftIO $ findExecutable cabalFmtExePath
   case x of
     Just _ -> do
@@ -85,7 +86,7 @@ provider recorder plId ideState _ FormatText contents nfp opts = do
           pure $ InL fmtDiff
     Nothing -> do
       log Error $ LogFormatterBinNotFound cabalFmtExePath
-      throwError (PluginInternalError "No installation of cabal-gild could be found. Please install it globally, or provide the full path to the executable")
+      throwError (PluginInternalError "No installation of cabal-fmt could be found. Please install it globally, or provide the full path to the executable")
   where
     fp = fromNormalizedFilePath nfp
     tabularSize = opts ^. L.tabSize
