@@ -1,14 +1,26 @@
 {-# LANGUAGE CPP               #-}
+{-# LANGUAGE LambdaCase        #-}
 {-# LANGUAGE OverloadedStrings #-}
 module Main
   ( main
   ) where
 
+import           Ide.Logger
+import qualified Ide.Plugin.Cabal    as Cabal
 import qualified Ide.Plugin.CabalFmt as CabalFmt
 import           System.Directory    (findExecutable)
 import           System.FilePath
 import           Test.Hls
 
+data TestLog
+  = LogCabalFmt CabalFmt.Log
+  | LogCabal Cabal.Log
+
+instance Pretty TestLog where
+  pretty = \case
+    LogCabalFmt msg -> pretty msg
+    LogCabal msg -> pretty msg
+
 data CabalFmtFound = Found | NotFound
 
 isTestIsolated :: Bool
@@ -30,8 +42,11 @@ main = do
   foundCabalFmt <- isCabalFmtFound
   defaultTestRunner (tests foundCabalFmt)
 
-cabalFmtPlugin :: PluginTestDescriptor CabalFmt.Log
-cabalFmtPlugin = mkPluginTestDescriptor CabalFmt.descriptor "cabal-fmt"
+cabalFmtPlugin :: PluginTestDescriptor TestLog
+cabalFmtPlugin = mconcat
+  [ mkPluginTestDescriptor (CabalFmt.descriptor . cmapWithPrio LogCabalFmt) "cabal-fmt"
+  , mkPluginTestDescriptor (Cabal.descriptor . cmapWithPrio LogCabal) "cabal"
+  ]
 
 tests :: CabalFmtFound -> TestTree
 tests found = testGroup "cabal-fmt"
@@ -39,8 +54,9 @@ tests found = testGroup "cabal-fmt"
     cabalFmtGolden found "formats a simple document" "simple_testdata" "formatted_document" $ \doc -> do
       formatDoc doc (FormattingOptions 2 True Nothing Nothing Nothing)
 
-  , expectFailBecause "cabal-fmt can't expand modules if .cabal file is read from stdin. Tracking issue: https://github.com/phadej/cabal-fmt/pull/82" $
-    cabalFmtGolden found "formats a document with expand:src comment" "commented_testdata" "formatted_document" $ \doc -> do
+  -- TODO: cabal-fmt can't expand modules if .cabal file is read from stdin. Tracking
+  -- issue: https://github.com/phadej/cabal-fmt/pull/82
+  , cabalFmtGolden found "formats a document with expand:src comment" "commented_testdata" "formatted_document" $ \doc -> do
       formatDoc doc (FormattingOptions 2 True Nothing Nothing Nothing)
 
   , cabalFmtGolden found "formats a document with lib information" "lib_testdata" "formatted_document" $ \doc -> do
