@@ -8,11 +8,11 @@ import           Data.List.NonEmpty                            (NonEmpty)
 import qualified Data.List.NonEmpty                            as NE
 import qualified Data.Map                                      as Map
 import qualified Data.Text                                     as T
-import qualified Data.Text.Encoding                            as T
 import           Development.IDE                               as D
 import qualified Development.IDE.Plugin.Completions.Types      as Ghcide
 import qualified Distribution.Fields                           as Syntax
 import qualified Distribution.Parsec.Position                  as Syntax
+import           Ide.Plugin.Cabal.Completion.CabalFields
 import           Ide.Plugin.Cabal.Completion.Completer.Simple
 import           Ide.Plugin.Cabal.Completion.Completer.Snippet
 import           Ide.Plugin.Cabal.Completion.Completer.Types   (Completer)
@@ -140,6 +140,7 @@ findCursorContext cursor parentHistory prefixText fields =
     Just field@(Syntax.Field _ _) -> classifyFieldContext parentHistory cursor field
     Just section@(Syntax.Section _ args sectionFields)
       | inSameLineAsSectionName section -> (stanzaCtx, None) -- TODO: test whether keyword in same line is parsed correctly
+      | getFieldName section `elem` conditionalKeywords -> findCursorContext cursor parentHistory prefixText sectionFields -- Ignore if conditionals, they are not real sections
       | otherwise ->
           findCursorContext cursor
             (NE.cons (Syntax.positionCol (getAnnotation section) + 1, Stanza (getFieldName section) (getOptionalSectionName args)) parentHistory)
@@ -147,6 +148,7 @@ findCursorContext cursor parentHistory prefixText fields =
     where
         inSameLineAsSectionName section = Syntax.positionRow (getAnnotation section) == Syntax.positionRow cursor
         stanzaCtx = snd $ NE.head parentHistory
+        conditionalKeywords = ["if", "elif", "else"]
 
 -- | Finds the cursor's context, where the cursor is already found to be in a specific field
 --
@@ -177,57 +179,3 @@ classifyFieldContext ctx cursor field
 
     cursorColumn = Syntax.positionCol cursor
     fieldColumn = Syntax.positionCol (getAnnotation field)
-
--- ----------------------------------------------------------------
--- Cabal-syntax utilities I don't really want to write myself
--- ----------------------------------------------------------------
-
--- | Determine the context of a cursor position within a stack of stanza contexts
---
--- If the cursor is indented more than one of the stanzas in the stack
--- the respective stanza is returned if this is never the case, the toplevel stanza
--- in the stack is returned.
-findStanzaForColumn :: Int -> NonEmpty (Int, StanzaContext) -> (StanzaContext, FieldContext)
-findStanzaForColumn col ctx = case NE.uncons ctx of
-    ((_, stanza), Nothing) -> (stanza, None)
-    ((indentation, stanza), Just res)
-        | col < indentation -> findStanzaForColumn col res
-        | otherwise -> (stanza, None)
-
--- | Determine the field the cursor is currently a part of.
---
--- The result is said field and its starting position
--- or Nothing if the passed list of fields is empty.
-
--- This only looks at the row of the cursor and not at the cursor's
--- position within the row.
---
--- TODO: we do not handle braces correctly. Add more tests!
-findFieldSection :: Syntax.Position -> [Syntax.Field Syntax.Position] -> Maybe (Syntax.Field Syntax.Position)
-findFieldSection _cursor [] = Nothing
-findFieldSection _cursor [x] =
-  -- Last field. We decide later, whether we are starting
-  -- a new section.
-  Just x
-findFieldSection cursor (x:y:ys)
-    | Syntax.positionRow (getAnnotation x) <= cursorLine && cursorLine < Syntax.positionRow (getAnnotation y)
-    = Just x
-    | otherwise = findFieldSection cursor (y:ys)
-  where
-    cursorLine = Syntax.positionRow cursor
-
-type FieldName = T.Text
-
-getAnnotation :: Syntax.Field ann -> ann
-getAnnotation (Syntax.Field (Syntax.Name ann _) _)     = ann
-getAnnotation (Syntax.Section (Syntax.Name ann _) _ _) = ann
-
-getFieldName :: Syntax.Field ann -> FieldName
-getFieldName (Syntax.Field (Syntax.Name _ fn) _)     = T.decodeUtf8 fn
-getFieldName (Syntax.Section (Syntax.Name _ fn) _ _) = T.decodeUtf8 fn
-
-getOptionalSectionName :: [Syntax.SectionArg ann] -> Maybe T.Text
-getOptionalSectionName [] = Nothing
-getOptionalSectionName (x:xs) = case x of
-    Syntax.SecArgName _ name -> Just (T.decodeUtf8 name)
-    _                        -> getOptionalSectionName xs
