@@ -8,7 +8,6 @@ import           Control.DeepSeq                 (NFData)
 import           Control.Lens                    ((^.))
 import           Data.Hashable
 import qualified Data.Text                       as T
-import           Data.Typeable
 import           Development.IDE                 as D
 import qualified Distribution.Fields             as Syntax
 import qualified Distribution.PackageDescription as PD
@@ -44,7 +43,7 @@ instance Pretty Log where
 type instance RuleResult ParseCabalFile = PD.GenericPackageDescription
 
 data ParseCabalFile = ParseCabalFile
-  deriving (Eq, Show, Typeable, Generic)
+  deriving (Eq, Show, Generic)
 
 instance Hashable ParseCabalFile
 
@@ -53,12 +52,21 @@ instance NFData ParseCabalFile
 type instance RuleResult ParseCabalFields = [Syntax.Field Syntax.Position]
 
 data ParseCabalFields = ParseCabalFields
-  deriving (Eq, Show, Typeable, Generic)
+  deriving (Eq, Show, Generic)
 
 instance Hashable ParseCabalFields
 
 instance NFData ParseCabalFields
 
+type instance RuleResult ParseCabalCommonSections = [Syntax.Field Syntax.Position]
+
+data ParseCabalCommonSections = ParseCabalCommonSections
+  deriving (Eq, Show, Generic)
+
+instance Hashable ParseCabalCommonSections
+
+instance NFData ParseCabalCommonSections
+
 -- | The context a cursor can be in within a cabal file.
 --
 --  We can be in stanzas or the top level,
@@ -171,3 +179,10 @@ lspPositionToCabalPosition :: Position -> Syntax.Position
 lspPositionToCabalPosition pos = Syntax.Position
   (fromIntegral (pos ^. JL.line) + 1)
   (fromIntegral (pos ^. JL.character) + 1)
+
+-- | Convert an 'Syntax.Position' to a LSP 'Position'.
+--
+-- Cabal Positions start their indexing at 1 while LSP starts at 0.
+-- This helper makes sure, the translation is done properly.
+cabalPositionToLSPPosition :: Syntax.Position -> Position
+cabalPositionToLSPPosition (Syntax.Position start end) = Position (toEnum start -1) (toEnum end -1)
