@@ -7,6 +7,7 @@ module Completer where
 
 import           Control.Lens                                   ((^.), (^?))
 import           Control.Lens.Prism
+import           Control.Monad                                  (forM_)
 import qualified Data.ByteString                                as ByteString
 import qualified Data.ByteString.Char8                          as BS8
 import           Data.Maybe                                     (mapMaybe)
@@ -40,7 +41,8 @@ completerTests =
       completionHelperTests,
       filePathExposedModulesTests,
       exposedModuleCompleterTests,
-      importCompleterTests
+      importCompleterTests,
+      autogenFieldCompletionTests
     ]
 
 basicCompleterTests :: TestTree
@@ -336,6 +338,26 @@ importCompleterTests =
           [Syntax.SecArgName (Syntax.Position row (col + 7)) (BS8.pack name)]
           []
 
+autogenFieldCompletionTests :: TestTree
+autogenFieldCompletionTests =
+  testGroup "Autogen Field Completer Tests"
+    [ testAutogenField "library"         "completion/autogen-completion.cabal" (Position 6 9)  ["autogen-modules:", "autogen-includes:"]
+    , testAutogenField "executable"      "completion/autogen-completion.cabal" (Position 11 9) ["autogen-modules:", "autogen-includes:"]
+    , testAutogenField "test-suite"      "completion/autogen-completion.cabal" (Position 16 9) ["autogen-modules:", "autogen-includes:"]
+    , testAutogenField "benchmark"       "completion/autogen-completion.cabal" (Position 21 9) ["autogen-modules:", "autogen-includes:"]
+    , testAutogenField "common"          "completion/autogen-completion.cabal" (Position 24 9) ["autogen-modules:", "autogen-includes:"]
+    ]
+
+  where
+    testAutogenField :: String -> FilePath -> Position -> [T.Text] -> TestTree
+    testAutogenField section file pos expected = runCabalTestCaseSession ("autogen-modules completion in " <> section) "" $ do
+      doc <- openDoc file "cabal"
+      items <- getCompletions doc pos
+      let labels = map (^. L.label) items
+      liftIO $ forM_ expected $ \expect ->
+        assertBool (T.unpack expect <> " not found in " <> section) $
+          any (expect `T.isInfixOf`) labels
+
 simpleCompleterData :: Maybe StanzaName -> FilePath -> T.Text -> CompleterData
 simpleCompleterData sName dir pref = do
   CompleterData
@@ -370,39 +392,39 @@ extract item = case item ^. L.textEdit of
   _            -> error ""
 
 importTestData :: T.Text
-importTestData = [trimming|
-cabal-version:      3.0
-name:               hls-cabal-plugin
-version:            0.1.0.0
-synopsis:
-homepage:
-license:            MIT
-license-file:       LICENSE
-author:             Fendor
-maintainer:         fendor@posteo.de
-category:           Development
-extra-source-files: CHANGELOG.md
+importTestData = [__i|
+  cabal-version:      3.0
+  name:               hls-cabal-plugin
+  version:            0.1.0.0
+  synopsis:
+  homepage:
+  license:            MIT
+  license-file:       LICENSE
+  author:             Fendor
+  maintainer:         fendor@posteo.de
+  category:           Development
+  extra-source-files: CHANGELOG.md
 
-common defaults
-  default-language: GHC2021
-  -- Should have been in GHC2021, an oversight
-  default-extensions: ExplicitNamespaces
+  common defaults
+    default-language: GHC2021
+    -- Should have been in GHC2021, an oversight
+    default-extensions: ExplicitNamespaces
 
-common test-defaults
-  ghc-options: -threaded -rtsopts -with-rtsopts=-N
+  common test-defaults
+    ghc-options: -threaded -rtsopts -with-rtsopts=-N
 
-library
+  library
+      import:
+              ^
+      exposed-modules:  IDE.Plugin.Cabal
+      build-depends:    base ^>=4.14.3.0
+      hs-source-dirs:   src
+      default-language: Haskell2010
+
+  common notForLib
+    default-language: GHC2021
+
+  test-suite tests
     import:
             ^
-    exposed-modules:  IDE.Plugin.Cabal
-    build-depends:    base ^>=4.14.3.0
-    hs-source-dirs:   src
-    default-language: Haskell2010
-
-common notForLib
-  default-language: GHC2021
-
-test-suite tests
-  import:
-          ^
 |]
