@@ -12,11 +12,12 @@ module Development.IDE.Plugin.CodeAction
     fillHolePluginDescriptor,
     extendImportPluginDescriptor,
     -- * For testing
-    matchRegExMultipleImports
+    matchRegExMultipleImports,
+    extractNotInScopeName,
+    NotInScope(..)
     ) where
 
 import           Control.Applicative                               ((<|>))
-import           Control.Applicative.Combinators.NonEmpty          (sepBy1)
 import           Control.Arrow                                     (second,
                                                                     (&&&),
                                                                     (>>>))
@@ -48,7 +49,9 @@ import           Development.IDE.Core.Service
 import           Development.IDE.Core.Shake                        hiding (Log)
 import           Development.IDE.GHC.Compat                        hiding
                                                                    (ImplicitPrelude)
+#if !MIN_VERSION_ghc(9,11,0)
 import           Development.IDE.GHC.Compat.Util
+#endif
 import           Development.IDE.GHC.Error
 import           Development.IDE.GHC.ExactPrint
 import qualified Development.IDE.GHC.ExactPrint                    as E
@@ -69,16 +72,15 @@ import           Development.IDE.Types.Diagnostics
 import           Development.IDE.Types.Exports
 import           Development.IDE.Types.Location
 import           Development.IDE.Types.Options
-import           GHC                                               (AddEpAnn (AddEpAnn),
-                                                                    AnnsModule (am_main),
-                                                                    DeltaPos (..),
+import           GHC                                               (DeltaPos (..),
                                                                     EpAnn (..),
                                                                     LEpaComment)
+import           GHC.Iface.Ext.Types                               (ContextInfo (..),
+                                                                    IdentifierDetails (..))
 import qualified GHC.LanguageExtensions                            as Lang
 import           Ide.Logger                                        hiding
                                                                    (group)
 import           Ide.PluginUtils                                   (extendToFullLines,
-                                                                    extractTextInRange,
                                                                     subRange)
 import           Ide.Types
 import           Language.LSP.Protocol.Message                     (Method (..),
@@ -98,25 +100,36 @@ import           Language.LSP.Protocol.Types                       (ApplyWorkspa
                                                                     type (|?) (InL, InR),
                                                                     uriToFilePath)
 import qualified Text.Fuzzy.Parallel                               as TFP
-import qualified Text.Regex.Applicative                            as RE
 import           Text.Regex.TDFA                                   ((=~), (=~~))
 
 -- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
 
 #if !MIN_VERSION_ghc(9,9,0)
 import           Development.IDE.GHC.Compat.ExactPrint             (makeDeltaAst)
-import           GHC                                               (Anchor (anchor_op),
+import           GHC                                               (AddEpAnn (AddEpAnn),
+                                                                    Anchor (anchor_op),
                                                                     AnchorOperation (..),
+                                                                    AnnsModule (am_main),
                                                                     EpaLocation (..))
 #endif
 
-#if MIN_VERSION_ghc(9,9,0)
-import           GHC                                               (EpaLocation,
+#if MIN_VERSION_ghc(9,9,0) && !MIN_VERSION_ghc(9,11,0)
+import           GHC                                               (AddEpAnn (AddEpAnn),
+                                                                    AnnsModule (am_main),
+                                                                    EpaLocation,
                                                                     EpaLocation' (..),
                                                                     HasLoc (..))
-import           GHC.Types.SrcLoc                                  (srcSpanToRealSrcSpan)
 #endif
 
+#if MIN_VERSION_ghc(9,11,0)
+import           GHC                                               (AnnsModule (am_where),
+                                                                    EpToken (..),
+                                                                    EpaLocation,
+                                                                    EpaLocation' (..),
+                                                                    HasLoc (..))
+#endif
+
+
 -------------------------------------------------------------------------------------------------
 
 -- | Generate code actions.
@@ -256,19 +269,11 @@ extendImportHandler' ideState ExtendImport {..}
 
 isWantedModule :: ModuleName -> Maybe ModuleName -> GenLocated l (ImportDecl GhcPs) -> Bool
 isWantedModule wantedModule Nothing (L _ it@ImportDecl{ ideclName
-#if MIN_VERSION_ghc(9,5,0)
                                                       , ideclImportList = Just (Exactly, _)
-#else
-                                                      , ideclHiding = Just (False, _)
-#endif
                                                       }) =
     not (isQualifiedImport it) && unLoc ideclName == wantedModule
 isWantedModule wantedModule (Just qual) (L _ ImportDecl{ ideclAs, ideclName
-#if MIN_VERSION_ghc(9,5,0)
                                                        , ideclImportList = Just (Exactly, _)
-#else
-                                                       , ideclHiding = Just (False, _)
-#endif
                                                        }) =
     unLoc ideclName == wantedModule && (wantedModule == qual || (unLoc <$> ideclAs) == Just qual)
 isWantedModule _ _ _ = False
@@ -339,7 +344,11 @@ findSigOfBinds range = go
         case unLoc <$> findDeclContainingLoc (_start range) lsigs of
           Just sig' -> Just sig'
           Nothing -> do
+#if MIN_VERSION_ghc(9,11,0)
+            lHsBindLR <- findDeclContainingLoc (_start range) binds
+#else
             lHsBindLR <- findDeclContainingLoc (_start range) (bagToList binds)
+#endif
             findSigOfBind range (unLoc lHsBindLR)
     go _ = Nothing
 
@@ -420,7 +429,11 @@ isUnusedImportedId
   modName
   importSpan
     | occ <- mkVarOcc identifier,
+#if MIN_VERSION_ghc(9,11,0)
+      impModsVals <- importedByUser . concat $ M.elems imp_mods,
+#else
       impModsVals <- importedByUser . concat $ moduleEnvElts imp_mods,
+#endif
       Just rdrEnv <-
         listToMaybe
           [ imv_all_exports
@@ -658,10 +671,16 @@ suggestDeleteUnusedBinding
         indexedContent
         name
         (L _ Match{m_grhss=GRHSs{grhssLocalBinds}}) = do
-        let go bag lsigs =
-                if isEmptyBag bag
-                then []
-                else concatMap (findRelatedSpanForHsBind indexedContent name lsigs) bag
+        let emptyBag bag =
+#if MIN_VERSION_ghc(9,11,0)
+                null bag
+#else
+                isEmptyBag bag
+#endif
+            go bag lsigs =
+                  if emptyBag bag
+                  then []
+                  else concatMap (findRelatedSpanForHsBind indexedContent name lsigs) bag
         case grhssLocalBinds of
           (HsValBinds _ (ValBinds _ bag lsigs)) -> go bag lsigs
           _                                     -> []
@@ -832,7 +851,6 @@ suggestAddTypeAnnotationToSatisfyConstraints sourceOpt Diagnostic{_range=_range,
     | otherwise = []
     where
       makeAnnotatedLit ty lit = "(" <> lit <> " :: " <> ty <> ")"
-#if MIN_VERSION_ghc(9,4,0)
       pat multiple at inArg inExpr = T.concat [ ".*Defaulting the type variable "
                                        , ".*to type ‘([^ ]+)’ "
                                        , "in the following constraint"
@@ -843,17 +861,6 @@ suggestAddTypeAnnotationToSatisfyConstraints sourceOpt Diagnostic{_range=_range,
                                        , if inExpr then ".+In the expression" else ""
                                        , ".+In the expression"
                                        ]
-#else
-      pat multiple at inArg inExpr = T.concat [ ".*Defaulting the following constraint"
-                                       , if multiple then "s" else ""
-                                       , " to type ‘([^ ]+)’ "
-                                       , ".*arising from the literal ‘(.+)’"
-                                       , if inArg then ".+In the.+argument" else ""
-                                       , if at then ".+at ([^ ]*)" else ""
-                                       , if inExpr then ".+In the expression" else ""
-                                       , ".+In the expression"
-                                       ]
-#endif
       codeEdit range ty lit replacement =
         let title = "Add type annotation ‘" <> ty <> "’ to ‘" <> lit <> "’"
             edits = [TextEdit range replacement]
@@ -1133,17 +1140,10 @@ occursUnqualified symbol ImportDecl{..}
     | isNothing ideclAs = Just False /=
             -- I don't find this particularly comprehensible,
             -- but HLint suggested me to do so...
-#if MIN_VERSION_ghc(9,5,0)
         (ideclImportList <&> \(isHiding, L _ ents) ->
             let occurs = any ((symbol `symbolOccursIn`) . unLoc) ents
             in (isHiding == EverythingBut) && not occurs || (isHiding == Exactly) && occurs
         )
-#else
-        (ideclHiding <&> \(isHiding, L _ ents) ->
-            let occurs = any ((symbol `symbolOccursIn`) . unLoc) ents
-            in isHiding && not occurs || not isHiding && occurs
-        )
-#endif
 occursUnqualified _ _ = False
 
 symbolOccursIn :: T.Text -> IE GhcPs -> Bool
@@ -1481,11 +1481,6 @@ suggestNewImport df packageExportsMap ps fileContents Diagnostic{..}
         >>= (findImportDeclByModuleName hsmodImports . T.unpack)
         >>= ideclAs . unLoc
         <&> T.pack . moduleNameString . unLoc
-  , -- tentative workaround for detecting qualification in GHC 9.4
-    -- FIXME: We can delete this after dropping the support for GHC 9.4
-    qualGHC94 <-
-        guard (ghcVersion == GHC94)
-            *> extractQualifiedModuleNameFromMissingName (extractTextInRange _range fileContents)
   , Just (range, indent) <- newImportInsertRange ps fileContents
   , extendImportSuggestions <- matchRegexUnifySpaces msg
 #if MIN_VERSION_ghc(9,7,0)
@@ -1494,82 +1489,32 @@ suggestNewImport df packageExportsMap ps fileContents Diagnostic{..}
     "Perhaps you want to add ‘[^’]*’ to the import list in the import of ‘([^’]*)’"
 #endif
   = let qis = qualifiedImportStyle df
-        -- FIXME: we can use thingMissing once the support for GHC 9.4 is dropped.
-        -- In what fllows, @missing@ is assumed to be qualified name.
-        -- @thingMissing@ is already as desired with GHC != 9.4.
-        -- In GHC 9.4, however, GHC drops a module qualifier from a qualified symbol.
-        -- Thus we need to explicitly concatenate qualifier explicity in GHC 9.4.
-        missing
-            | GHC94 <- ghcVersion
-            , isNothing (qual <|> qual')
-            , Just q <- qualGHC94 =
-                qualify q thingMissing
-            | otherwise = thingMissing
         suggestions = nubSortBy simpleCompareImportSuggestion
-          (constructNewImportSuggestions packageExportsMap (qual <|> qual' <|> qualGHC94, missing) extendImportSuggestions qis) in
+          (constructNewImportSuggestions packageExportsMap (qual <|> qual', thingMissing) extendImportSuggestions qis) in
     map (\(ImportSuggestion _ kind (unNewImport -> imp)) -> (imp, kind, TextEdit range (imp <> "\n" <> T.replicate indent " "))) suggestions
   where
-    qualify q (NotInScopeDataConstructor d) = NotInScopeDataConstructor (q <> "." <> d)
-    qualify q (NotInScopeTypeConstructorOrClass d) = NotInScopeTypeConstructorOrClass (q <> "." <> d)
-    qualify q (NotInScopeThing d) = NotInScopeThing (q <> "." <> d)
-
     L _ HsModule {..} = ps
 suggestNewImport _ _ _ _ _ = []
 
-{- |
-Extracts qualifier of the symbol from the missing symbol.
-Input must be either a plain qualified variable or possibly-parenthesized qualified binary operator (though no strict checking is done for symbol part).
-This is only needed to alleviate the issue #3473.
-
-FIXME: We can delete this after dropping the support for GHC 9.4
-
->>> extractQualifiedModuleNameFromMissingName "P.lookup"
-Just "P"
-
->>> extractQualifiedModuleNameFromMissingName "ΣP3_'.σlookup"
-Just "\931P3_'"
-
->>> extractQualifiedModuleNameFromMissingName "ModuleA.Gre_ekσ.goodδ"
-Just "ModuleA.Gre_ek\963"
-
->>> extractQualifiedModuleNameFromMissingName "(ModuleA.Gre_ekσ.+)"
-Just "ModuleA.Gre_ek\963"
-
->>> extractQualifiedModuleNameFromMissingName "(ModuleA.Gre_ekσ..|.)"
-Just "ModuleA.Gre_ek\963"
-
->>> extractQualifiedModuleNameFromMissingName "A.B.|."
-Just "A.B"
--}
-extractQualifiedModuleNameFromMissingName :: T.Text -> Maybe T.Text
-extractQualifiedModuleNameFromMissingName (T.strip -> missing)
-    = T.pack <$> (T.unpack missing RE.=~ qualIdentP)
-    where
-        {-
-        NOTE: Haskell 2010 allows /unicode/ upper & lower letters
-        as a module name component; otoh, regex-tdfa only allows
-        /ASCII/ letters to be matched with @[[:upper:]]@ and/or @[[:lower:]]@.
-        Hence we use regex-applicative(-text) for finer-grained predicates.
-
-        RULES (from [Section 10 of Haskell 2010 Report](https://www.haskell.org/onlinereport/haskell2010/haskellch10.html)):
-            modid	→	{conid .} conid
-            conid	→	large {small | large | digit | ' }
-            small	→	ascSmall | uniSmall | _
-            ascSmall	→	a | b | … | z
-            uniSmall	→	any Unicode lowercase letter
-            large	→	ascLarge | uniLarge
-            ascLarge	→	A | B | … | Z
-            uniLarge	→	any uppercase or titlecase Unicode letter
-        -}
-
-        qualIdentP = parensQualOpP <|> qualVarP
-        parensQualOpP = RE.sym '(' *> modNameP <* RE.sym '.' <* RE.anySym <* RE.few RE.anySym <* RE.sym ')'
-        qualVarP = modNameP <* RE.sym '.' <* RE.some RE.anySym
-        conIDP = RE.withMatched $
-            RE.psym isUpper
-            *> RE.many
-                (RE.psym $ \c -> c == '\'' || c == '_' || isUpper c || isLower c || isDigit c)
-        modNameP = fmap snd $ RE.withMatched $ conIDP `sepBy1` RE.sym '.'
+-- | A Backward compatible implementation of `lookupOccEnv_AllNameSpaces` for
+-- GHC <=9.6
+--
+-- It looks for a symbol name in all known namespaces, including types,
+-- variables, and fieldnames.
+--
+-- Note that on GHC >= 9.8, the record selectors are not in the `mkVarOrDataOcc`
+-- anymore, but are in a custom namespace, see
+-- https://gitlab.haskell.org/ghc/ghc/-/wikis/migration/9.8#new-namespace-for-record-fields,
+-- hence we need to use this "AllNamespaces" implementation, otherwise we'll
+-- miss them.
+lookupOccEnvAllNamespaces :: ExportsMap -> T.Text -> [IdentInfo]
+#if MIN_VERSION_ghc(9,7,0)
+lookupOccEnvAllNamespaces exportsMap name = Set.toList $ mconcat (lookupOccEnv_AllNameSpaces (getExportsMap exportsMap) (mkTypeOcc name))
+#else
+lookupOccEnvAllNamespaces exportsMap name = maybe [] Set.toList $
+                            lookupOccEnv (getExportsMap exportsMap) (mkVarOrDataOcc name)
+                          <> lookupOccEnv (getExportsMap exportsMap) (mkTypeOcc name) -- look up the modified unknown name in the export map
+#endif
 
 
 constructNewImportSuggestions
@@ -1577,8 +1522,8 @@ constructNewImportSuggestions
 constructNewImportSuggestions exportsMap (qual, thingMissing) notTheseModules qis = nubOrdBy simpleCompareImportSuggestion
   [ suggestion
   | Just name <- [T.stripPrefix (maybe "" (<> ".") qual) $ notInScope thingMissing] -- strip away qualified module names from the unknown name
-  , identInfo <- maybe [] Set.toList $ lookupOccEnv (getExportsMap exportsMap) (mkVarOrDataOcc name)
-                                    <> lookupOccEnv (getExportsMap exportsMap) (mkTypeOcc name) -- look up the modified unknown name in the export map
+
+  , identInfo <- lookupOccEnvAllNamespaces exportsMap name -- look up the modified unknown name in the export map
   , canUseIdent thingMissing identInfo                                              -- check if the identifier information retrieved can be used
   , moduleNameText identInfo `notElem` fromMaybe [] notTheseModules                 -- check if the module of the identifier is allowed
   , suggestion <- renderNewImport identInfo                                         -- creates a list of import suggestions for the retrieved identifier information
@@ -1693,38 +1638,47 @@ findPositionAfterModuleName ps _hsmodName' = do
     -- The relative position of 'where' keyword (in lines, relative to the previous AST node).
     -- The exact-print API changed a lot in ghc-9.2, so we need to handle it separately for different compiler versions.
     whereKeywordLineOffset :: Maybe Int
-#if MIN_VERSION_ghc(9,5,0)
     whereKeywordLineOffset = case hsmodAnn hsmodExt of
-#else
-    whereKeywordLineOffset = case hsmodAnn of
-#endif
         EpAnn _ annsModule _ -> do
             -- Find the first 'where'
+#if MIN_VERSION_ghc(9,11,0)
+            whereLocation <- filterWhere $ am_where annsModule
+#else
             whereLocation <- listToMaybe . mapMaybe filterWhere $ am_main annsModule
+#endif
             epaLocationToLine whereLocation
 #if !MIN_VERSION_ghc(9,9,0)
         EpAnnNotUsed -> Nothing
 #endif
+#if MIN_VERSION_ghc(9,11,0)
+    filterWhere (EpTok loc)             = Just loc
+    filterWhere _                       = Nothing
+#else
     filterWhere (AddEpAnn AnnWhere loc) = Just loc
     filterWhere _                       = Nothing
+#endif
 
     epaLocationToLine :: EpaLocation -> Maybe Int
 #if MIN_VERSION_ghc(9,9,0)
     epaLocationToLine (EpaSpan sp)
       = fmap (srcLocLine . realSrcSpanEnd) $ srcSpanToRealSrcSpan sp
-#elif MIN_VERSION_ghc(9,5,0)
+#else
     epaLocationToLine (EpaSpan sp _)
       = Just . srcLocLine . realSrcSpanEnd $ sp
-#else
-    epaLocationToLine (EpaSpan sp)
-      = Just . srcLocLine . realSrcSpanEnd $ sp
 #endif
+#if MIN_VERSION_ghc(9,11,0)
+    epaLocationToLine (EpaDelta _ (SameLine _) priorComments) = Just $ sumCommentsOffset priorComments
+    -- 'priorComments' contains the comments right before the current EpaLocation
+    -- Summing line offset of priorComments is necessary, as 'line' is the gap between the last comment and
+    -- the current AST node
+    epaLocationToLine (EpaDelta _ (DifferentLine line _) priorComments) = Just (line + sumCommentsOffset priorComments)
+#else
     epaLocationToLine (EpaDelta (SameLine _) priorComments) = Just $ sumCommentsOffset priorComments
     -- 'priorComments' contains the comments right before the current EpaLocation
     -- Summing line offset of priorComments is necessary, as 'line' is the gap between the last comment and
     -- the current AST node
     epaLocationToLine (EpaDelta (DifferentLine line _) priorComments) = Just (line + sumCommentsOffset priorComments)
-
+#endif
     sumCommentsOffset :: [LEpaComment] -> Int
 #if MIN_VERSION_ghc(9,9,0)
     sumCommentsOffset = sum . fmap (\(L anchor _) -> anchorOpLine anchor)
@@ -1732,7 +1686,12 @@ findPositionAfterModuleName ps _hsmodName' = do
     sumCommentsOffset = sum . fmap (\(L anchor _) -> anchorOpLine (anchor_op anchor))
 #endif
 
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+    anchorOpLine :: EpaLocation' a -> Int
+    anchorOpLine EpaSpan{}                             = 0
+    anchorOpLine (EpaDelta _ (SameLine _) _)           = 0
+    anchorOpLine (EpaDelta _ (DifferentLine line _) _) = line
+#elif MIN_VERSION_ghc(9,9,0)
     anchorOpLine :: EpaLocation' a -> Int
     anchorOpLine EpaSpan{}                           = 0
     anchorOpLine (EpaDelta (SameLine _) _)           = 0
@@ -1825,7 +1784,7 @@ data NotInScope
     = NotInScopeDataConstructor T.Text
     | NotInScopeTypeConstructorOrClass T.Text
     | NotInScopeThing T.Text
-    deriving Show
+    deriving (Show, Eq)
 
 notInScope :: NotInScope -> T.Text
 notInScope (NotInScopeDataConstructor t)        = t
@@ -1840,6 +1799,38 @@ extractNotInScopeName x
   = Just $ NotInScopeDataConstructor name
   | Just [name] <- matchRegexUnifySpaces x "ot in scope: type constructor or class [^‘]*‘([^’]*)’"
   = Just $ NotInScopeTypeConstructorOrClass name
+  | Just [name] <- matchRegexUnifySpaces x "The data constructors of ‘([^ ]+)’ are not all in scope"
+  = Just $ NotInScopeDataConstructor name
+  | Just [name] <- matchRegexUnifySpaces x "of newtype ‘([^’]*)’ is not in scope"
+  = Just $ NotInScopeThing name
+  -- Match for HasField "foo" Bar String in the context where, e.g. x.foo is
+  -- used, and x :: Bar.
+  --
+  -- This usually mean that the field is not in scope and the correct fix is to
+  -- import (Bar(foo)) or (Bar(..)).
+  --
+  -- However, it is more reliable to match for the type name instead of the field
+  -- name, and most of the time you'll want to import the complete type with all
+  -- their fields instead of the specific field.
+  --
+  -- The regex is convoluted because it accounts for:
+  --
+  -- - Qualified (or not) `HasField`
+  -- - The type bar is always qualified. If it is unqualified, it means that the
+  -- parent module is already imported, and in this context it uses an hint
+  -- already available in the GHC error message. However this regex accounts for
+  -- qualified or not, it does not cost much and should be more robust if the
+  -- hint changes in the future
+  -- - Next regex will account for polymorphic types, which appears as `HasField
+  -- "foo" (Bar Int)...`, e.g. see the parenthesis
+  | Just [_module, name] <- matchRegexUnifySpaces x "No instance for [‘(].*HasField \"[^\"]+\" ([^ (.]+\\.)*([^ (.]+).*[’)]"
+  = Just $ NotInScopeThing name
+  | Just [_module, name] <- matchRegexUnifySpaces x "No instance for [‘(].*HasField \"[^\"]+\" \\(([^ .]+\\.)*([^ .]+)[^)]*\\).*[’)]"
+  = Just $ NotInScopeThing name
+  -- The order of the "Not in scope" is important, for example, some of the
+  -- matcher may catch the "record" value instead of the value later.
+  | Just [name] <- matchRegexUnifySpaces x "Not in scope: record field ‘([^’]*)’"
+  = Just $ NotInScopeThing name
   | Just [name] <- matchRegexUnifySpaces x "ot in scope: \\(([^‘ ]+)\\)"
   = Just $ NotInScopeThing name
   | Just [name] <- matchRegexUnifySpaces x "ot in scope: ([^‘ ]+)"
@@ -1881,14 +1872,11 @@ extractQualifiedModuleName x
 --         ‘Data.Functor’ nor ‘Data.Text’ exports ‘putStrLn’.
 extractDoesNotExportModuleName :: T.Text -> Maybe T.Text
 extractDoesNotExportModuleName x
-  | Just [m] <-
-#if MIN_VERSION_ghc(9,4,0)
-    matchRegexUnifySpaces x "the module ‘([^’]*)’ does not export"
-      <|> matchRegexUnifySpaces x "nor ‘([^’]*)’ export"
-#else
-    matchRegexUnifySpaces x "Module ‘([^’]*)’ does not export"
-      <|> matchRegexUnifySpaces x "nor ‘([^’]*)’ exports"
-#endif
+  | Just [m] <- case ghcVersion of
+                  GHC912 -> matchRegexUnifySpaces x "The module ‘([^’]*)’ does not export"
+                            <|> matchRegexUnifySpaces x "nor ‘([^’]*)’ export"
+                  _ ->      matchRegexUnifySpaces x "the module ‘([^’]*)’ does not export"
+                            <|> matchRegexUnifySpaces x "nor ‘([^’]*)’ export"
   = Just m
   | otherwise
   = Nothing
@@ -1959,21 +1947,12 @@ textInRange (Range (Position (fromIntegral -> startRow) (fromIntegral -> startCo
 
 -- | Returns the ranges for a binding in an import declaration
 rangesForBindingImport :: ImportDecl GhcPs -> String -> [Range]
-#if MIN_VERSION_ghc(9,5,0)
 rangesForBindingImport ImportDecl{
   ideclImportList = Just (Exactly, L _ lies)
   } b =
     concatMap (mapMaybe srcSpanToRange . rangesForBinding' b') lies
   where
     b' = wrapOperatorInParens b
-#else
-rangesForBindingImport ImportDecl{
-  ideclHiding = Just (False, L _ lies)
-  } b =
-    concatMap (mapMaybe srcSpanToRange . rangesForBinding' b') lies
-  where
-    b' = wrapOperatorInParens b
-#endif
 rangesForBindingImport _ _ = []
 
 wrapOperatorInParens :: String -> String
