@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE GADTs        #-}
 {-# LANGUAGE TypeFamilies #-}
 module Development.IDE.Plugin.CodeAction.ExactPrint (
@@ -35,10 +36,8 @@ import           Control.Lens                           (_head, _last, over)
 import           Data.Bifunctor                         (first)
 import           Data.Maybe                             (fromMaybe, mapMaybe)
 import           Development.IDE.Plugin.CodeAction.Util
-import           GHC                                    (AddEpAnn (..),
-                                                         AnnContext (..),
+import           GHC                                    (AnnContext (..),
                                                          AnnList (..),
-                                                         AnnParen (..),
                                                          DeltaPos (SameLine),
                                                          EpAnn (..),
                                                          IsUnicodeSyntax (NormalSyntax),
@@ -46,8 +45,17 @@ import           GHC                                    (AddEpAnn (..),
                                                          TrailingAnn (AddCommaAnn),
                                                          emptyComments, reAnnL)
 
+
 -- See Note [Guidelines For Using CPP In GHCIDE Import Statements]
 
+#if MIN_VERSION_ghc(9,11,0)
+import GHC (EpToken (..)
+           , AnnListBrackets (..)
+           , EpUniToken (..))
+#else
+import GHC (AddEpAnn (..),
+                                                         AnnParen (..))
+#endif
 #if !MIN_VERSION_ghc(9,9,0)
 import           Data.Default                           (Default (..))
 import           GHC                                    (addAnns, ann)
@@ -55,6 +63,7 @@ import           GHC                                    (addAnns, ann)
 
 #if MIN_VERSION_ghc(9,9,0)
 import           GHC                                    (NoAnn (..))
+import           GHC                                    (EpAnnComments (..))
 #endif
 
 ------------------------------------------------------------------------------
@@ -131,10 +140,8 @@ removeConstraint toRemove = go . traceAst "REMOVE_CONSTRAINT_input"
     go :: LHsType GhcPs -> Rewrite
 #if MIN_VERSION_ghc(9,9,0)
     go lHsType@(makeDeltaAst -> L l it@HsQualTy{hst_ctxt = L l' ctxt, hst_body}) = Rewrite (locA lHsType) $ \_ -> do
-#elif MIN_VERSION_ghc(9,4,0)
-    go (L l it@HsQualTy{hst_ctxt = L l' ctxt, hst_body}) = Rewrite (locA l) $ \_ -> do
 #else
-    go (L l it@HsQualTy{hst_ctxt = Just (L l' ctxt), hst_body}) = Rewrite (locA l) $ \_ -> do
+    go (L l it@HsQualTy{hst_ctxt = L l' ctxt, hst_body}) = Rewrite (locA l) $ \_ -> do
 #endif
       let ctxt' = filter (not . toRemove) ctxt
           removeStuff = (toRemove <$> headMaybe ctxt) == Just True
@@ -143,11 +150,7 @@ removeConstraint toRemove = go . traceAst "REMOVE_CONSTRAINT_input"
           [] -> hst_body'
           _ -> do
             let ctxt'' = over _last (first removeComma) ctxt'
-#if MIN_VERSION_ghc(9,4,0)
             L l $ it{ hst_ctxt = L l' ctxt''
-#else
-            L l $ it{ hst_ctxt = Just $ L l' ctxt''
-#endif
                     , hst_body = hst_body'
                     }
     go (L _ (HsParTy _ ty)) = go ty
@@ -164,33 +167,27 @@ appendConstraint ::
   Rewrite
 appendConstraint constraintT = go . traceAst "appendConstraint"
  where
-#if MIN_VERSION_ghc(9,4,0)
   go (L l it@HsQualTy{hst_ctxt = L l' ctxt}) = Rewrite (locA l) $ \df -> do
-#else
-  go (L l it@HsQualTy{hst_ctxt = Just (L l' ctxt)}) = Rewrite (locA l) $ \df -> do
-#endif
     constraint <- liftParseAST df constraintT
     constraint <- pure $ setEntryDP constraint (SameLine 1)
 #if MIN_VERSION_ghc(9,9,0)
-    let l'' = fmap (addParensToCtxt close_dp) l'
+    let l'' = moveCommentsToTheEnd $ fmap (addParensToCtxt close_dp) l'
 #else
     let l'' = (fmap.fmap) (addParensToCtxt close_dp) l'
 #endif
     -- For singleton constraints, the close Paren DP is attached to an HsPar wrapping the constraint
     -- we have to reposition it manually into the AnnContext
         close_dp = case ctxt of
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+            [L _ (HsParTy (_, (EpTok ap_close)) _)] -> Just ap_close
+#elif MIN_VERSION_ghc(9,9,0)
             [L _ (HsParTy AnnParen{ap_close} _)] -> Just ap_close
 #else
             [L _ (HsParTy EpAnn{anns=AnnParen{ap_close}} _)] -> Just ap_close
 #endif
             _ -> Nothing
         ctxt' = over _last (first addComma) $ map dropHsParTy ctxt
-#if MIN_VERSION_ghc(9,4,0)
     return $ L l $ it{hst_ctxt = L l'' $ ctxt' ++ [constraint]}
-#else
-    return $ L l $ it{hst_ctxt = Just $ L l'' $ ctxt' ++ [constraint]}
-#endif
   go (L _ HsForAllTy{hst_body}) = go hst_body
   go (L _ (HsParTy _ ty)) = go ty
   go ast@(L l _) = Rewrite (locA l) $ \df -> do
@@ -198,17 +195,37 @@ appendConstraint constraintT = go . traceAst "appendConstraint"
     constraint <- liftParseAST df constraintT
     lContext <- uniqueSrcSpanT
     lTop <- uniqueSrcSpanT
-#if MIN_VERSION_ghc(9,4,0)
     let context = reAnnL annCtxt emptyComments $ L lContext [resetEntryDP constraint]
+#if MIN_VERSION_ghc(9,11,0)
+        annCtxt = AnnContext (Just (EpUniTok (epl 1) NormalSyntax)) [EpTok (epl 0) | needsParens] [EpTok (epl 0) | needsParens]
 #else
-    let context = Just $ reAnnL annCtxt emptyComments $ L lContext [resetEntryDP constraint]
-#endif
         annCtxt = AnnContext (Just (NormalSyntax, epl 1)) [epl 0 | needsParens] [epl 0 | needsParens]
+#endif
         needsParens = hsTypeNeedsParens sigPrec $ unLoc constraint
     ast <- pure $ setEntryDP (makeDeltaAst ast) (SameLine 1)
 
     return $ reLocA $ L lTop $ HsQualTy noExtField context ast
 
+#if MIN_VERSION_ghc(9,9,0)
+-- | This moves comment annotation toward the end of the block
+-- This is useful when extending a block, so the comment correctly appears
+-- after.
+--
+-- See https://github.com/haskell/haskell-language-server/issues/4648 for
+-- discussion.
+--
+-- For example, the following element, @(Foo) => -- hello@, when introducing an
+-- additionnal constraint, `Bar`, instead of getting `@(Foo, Bar) => -- hello@,
+-- we get @(Foo, -- hello Bar) =>@
+--
+-- This is a bit painful that the pretty printer is not able to realize that it
+-- introduces the token `=>` inside the comment and instead does something with
+-- meaning, but that's another story.
+moveCommentsToTheEnd :: EpAnn ann -> EpAnn ann
+moveCommentsToTheEnd (EpAnn entry anns (EpaComments priors)) = EpAnn entry anns (EpaCommentsBalanced { priorComments = [], followingComments = priors})
+moveCommentsToTheEnd (EpAnn entry anns (EpaCommentsBalanced priors following)) = EpAnn entry anns (EpaCommentsBalanced { priorComments = [], followingComments = priors <> following})
+#endif
+
 liftParseAST
     :: forall ast l.  (ASTElement l ast, ExactPrint (LocatedAn l ast))
     => DynFlags -> String -> TransformT (Either String) (LocatedAn l ast)
@@ -250,11 +267,7 @@ extendImportTopLevel ::
   LImportDecl GhcPs ->
   TransformT (Either String) (LImportDecl GhcPs)
 extendImportTopLevel thing (L l it@ImportDecl{..})
-#if MIN_VERSION_ghc(9,5,0)
   | Just (hide, L l' lies) <- ideclImportList
-#else
-  | Just (hide, L l' lies) <- ideclHiding
-#endif
   = do
     src <- uniqueSrcSpanT
     top <- uniqueSrcSpanT
@@ -266,9 +279,7 @@ extendImportTopLevel thing (L l it@ImportDecl{..})
       TransformT $ lift (Left $ thing <> " already imported")
 
     let lie = reLocA $ L src $ IEName
-#if MIN_VERSION_ghc(9,5,0)
                                 noExtField
-#endif
                                 rdr
         x = reLocA $ L top $ IEVar
 #if MIN_VERSION_ghc(9,8,0)
@@ -285,11 +296,7 @@ extendImportTopLevel thing (L l it@ImportDecl{..})
       then TransformT $ lift (Left $ thing <> " already imported")
       else do
         let lies' = addCommaInImportList lies x
-#if MIN_VERSION_ghc(9,5,0)
         return $ L l it{ideclImportList = Just (hide, L l' lies')}
-#else
-        return $ L l it{ideclHiding = Just (hide, L l' lies')}
-#endif
 extendImportTopLevel _ _ = TransformT $ lift $ Left "Unable to extend the import list"
 
 wildCardSymbol :: String
@@ -319,11 +326,7 @@ extendImportViaParent ::
   LImportDecl GhcPs ->
   TransformT (Either String) (LImportDecl GhcPs)
 extendImportViaParent df parent child (L l it@ImportDecl{..})
-#if MIN_VERSION_ghc(9,5,0)
   | Just (hide, L l' lies) <- ideclImportList = go hide l' [] lies
-#else
-  | Just (hide, L l' lies) <- ideclHiding = go hide l' [] lies
-#endif
  where
 #if MIN_VERSION_ghc(9,9,0)
   go _hide _l' _pre ((L _ll' (IEThingAll _ (L _ ie) _)) : _xs)
@@ -341,12 +344,12 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
       srcChild <- uniqueSrcSpanT
       let childRdr = reLocA $ L srcChild $ mkRdrUnqual $ mkVarOcc child
           childLIE = reLocA $ L srcChild $ IEName
-#if MIN_VERSION_ghc(9,5,0)
                                              noExtField
-#endif
                                              childRdr
           x :: LIE GhcPs = L ll' $ IEThingWith
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+                                     (Nothing, (EpTok d1, NoEpTok, NoEpTok, EpTok noAnn))
+#elif MIN_VERSION_ghc(9,9,0)
                                      (Nothing, [AddEpAnn AnnOpenP d1, AddEpAnn AnnCloseP noAnn])
 #elif MIN_VERSION_ghc(9,7,0)
                                      (Nothing, addAnns mempty [AddEpAnn AnnOpenP d1, AddEpAnn AnnCloseP def] emptyComments)
@@ -358,12 +361,7 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
                                      docs
 #endif
 
-
-#if MIN_VERSION_ghc(9,5,0)
       return $ L l it{ideclImportList = Just (hide, L l' $ reverse pre ++ [x] ++ xs)}
-#else
-      return $ L l it{ideclHiding = Just (hide, L l' $ reverse pre ++ [x] ++ xs)}
-#endif
 
 #if MIN_VERSION_ghc(9,9,0)
   go hide l' pre ((L l'' (IEThingWith l''' twIE@(L _ ie) _ lies' docs)) : xs)
@@ -373,17 +371,15 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
     -- ThingWith ie lies' => ThingWith ie (lies' ++ [child])
     | parent == unIEWrappedName ie
     , child == wildCardSymbol = do
-#if MIN_VERSION_ghc(9,5,0)
         let it' = it{ideclImportList = Just (hide, lies)}
-#else
-        let it' = it{ideclHiding = Just (hide, lies)}
-#endif
             thing = IEThingWith newl twIE (IEWildcard 2) []
 #if MIN_VERSION_ghc(9,9,0)
                                 docs
 #endif
 #if MIN_VERSION_ghc(9,7,0) && !MIN_VERSION_ghc(9,9,0)
             newl = fmap (\ann -> ann ++ [AddEpAnn AnnDotdot d0]) <$> l'''
+#elif MIN_VERSION_ghc(9,11,0)
+            newl = (\(open, _, comma, close)  -> (open, EpTok d0, comma, close)) <$> l'''
 #else
             newl = (\ann -> ann ++ [AddEpAnn AnnDotdot d0]) <$> l'''
 #endif
@@ -401,15 +397,9 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
           TransformT $ lift (Left $ child <> " already included in " <> parent <> " imports")
 
         let childLIE = reLocA $ L srcChild $ IEName
-#if MIN_VERSION_ghc(9,5,0)
                                                noExtField
-#endif
                                                childRdr
-#if MIN_VERSION_ghc(9,5,0)
         let it' = it{ideclImportList = Just (hide, lies)}
-#else
-        let it' = it{ideclHiding = Just (hide, lies)}
-#endif
             lies = L l' $ reverse pre ++
                 [L l'' (IEThingWith l''' twIE NoIEWildcard (over _last fixLast lies' ++ [childLIE])
 #if MIN_VERSION_ghc(9,9,0)
@@ -427,21 +417,27 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
       parentRdr <- liftParseAST df parent
       let childRdr = reLocA $ L srcChild $ mkRdrUnqual $ mkVarOcc child
           isParentOperator = hasParen parent
+#if MIN_VERSION_ghc(9,11,0)
+      let parentLIE = reLocA $ L srcParent $ if isParentOperator then IEType (EpTok (epl 0)) parentRdr'
+#else
       let parentLIE = reLocA $ L srcParent $ if isParentOperator then IEType (epl 0) parentRdr'
-                                               else IEName
-#if MIN_VERSION_ghc(9,5,0)
-                                                      noExtField
 #endif
+                                               else IEName
+                                                      noExtField
                                                       parentRdr'
           parentRdr' = modifyAnns parentRdr $ \case
+#if MIN_VERSION_ghc(9,11,0)
+              it@NameAnn{nann_adornment = NameParens _ _} -> it{nann_adornment=NameParens (EpTok (epl 1)) (EpTok (epl 0))}
+#else
               it@NameAnn{nann_adornment = NameParens} -> it{nann_open = epl 1, nann_close = epl 0}
+#endif
               other -> other
           childLIE = reLocA $ L srcChild $ IEName
-#if MIN_VERSION_ghc(9,5,0)
                                              noExtField
-#endif
                                              childRdr
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+          listAnn = (Nothing, (EpTok (epl 1), NoEpTok, NoEpTok, EpTok (epl 0)))
+#elif MIN_VERSION_ghc(9,9,0)
           listAnn = (Nothing, [AddEpAnn AnnOpenP (epl 1), AddEpAnn AnnCloseP (epl 0)])
 #elif MIN_VERSION_ghc(9,7,0)
           listAnn = (Nothing, epAnn srcParent [AddEpAnn AnnOpenP (epl 1), AddEpAnn AnnCloseP (epl 0)])
@@ -454,11 +450,7 @@ extendImportViaParent df parent child (L l it@ImportDecl{..})
 #endif
 
           lies' = addCommaInImportList (reverse pre) x
-#if MIN_VERSION_ghc(9,5,0)
       return $ L l it{ideclImportList = Just (hide, L l' lies')}
-#else
-      return $ L l it{ideclHiding = Just (hide, L l' lies')}
-#endif
 extendImportViaParent _ _ _ _ = TransformT $ lift $ Left "Unable to extend the import list via parent"
 
 -- Add an item in an import list, taking care of adding comma if needed.
@@ -499,11 +491,7 @@ addCommaInImportList lies x =
     fixLast :: [LocatedAn AnnListItem a] -> [LocatedAn AnnListItem a]
     fixLast = over _last (first (if existingTrailingComma then id else addComma))
 
-#if MIN_VERSION_ghc(9,5,0)
 unIEWrappedName :: IEWrappedName GhcPs -> String
-#else
-unIEWrappedName :: IEWrappedName (IdP GhcPs) -> String
-#endif
 unIEWrappedName (occName -> occ) = T.unpack $ printOutputable $ parenSymOcc occ (ppr occ)
 
 hasParen :: String -> Bool
@@ -517,17 +505,10 @@ hasParen _         = False
 hideSymbol ::
   String -> LImportDecl GhcPs -> Rewrite
 hideSymbol symbol lidecl@(L loc ImportDecl{..}) =
-#if MIN_VERSION_ghc(9,5,0)
   case ideclImportList of
     Nothing -> Rewrite (locA loc) $ extendHiding symbol lidecl Nothing
     Just (EverythingBut, hides) -> Rewrite (locA loc) $ extendHiding symbol lidecl (Just hides)
     Just (Exactly, imports) -> Rewrite (locA loc) $ deleteFromImport symbol lidecl $ setEntryDP (makeDeltaAst imports) (SameLine 1)
-#else
-  case ideclHiding of
-    Nothing -> Rewrite (locA loc) $ extendHiding symbol lidecl Nothing
-    Just (True, hides) -> Rewrite (locA loc) $ extendHiding symbol lidecl (Just hides)
-    Just (False, imports) -> Rewrite (locA loc) $ deleteFromImport symbol lidecl imports
-#endif
 
 extendHiding ::
   String ->
@@ -538,7 +519,10 @@ extendHiding ::
 extendHiding symbol (L l idecls) mlies df = do
   L l' lies <- case mlies of
     Nothing -> do
-#if MIN_VERSION_ghc(9,9,0)
+#if MIN_VERSION_ghc(9,11,0)
+        let ann :: EpAnn (AnnList (EpToken "hiding", [EpToken ","]))
+            ann = noAnnSrcSpanDP0
+#elif MIN_VERSION_ghc(9,9,0)
         let ann = noAnnSrcSpanDP0
 #else
         src <- uniqueSrcSpanT
@@ -549,9 +533,14 @@ extendHiding symbol (L l idecls) mlies df = do
 #else
             ann' = flip (fmap.fmap) ann $ \x -> x
 #endif
+#if MIN_VERSION_ghc(9,11,0)
+                {al_rest = (EpTok (epl 1), [NoEpTok])
+                ,al_brackets=ListParens (EpTok (epl 1)) (EpTok (epl 0))
+#else
                 {al_rest = [AddEpAnn AnnHiding (epl 1)]
                 ,al_open = Just $ AddEpAnn AnnOpenP (epl 1)
                 ,al_close = Just $ AddEpAnn AnnCloseP (epl 0)
+#endif
                 }
         return $ L ann' []
     Just pr -> pure pr
@@ -561,9 +550,7 @@ extendHiding symbol (L l idecls) mlies df = do
   rdr <- liftParseAST df symbol
   rdr <- pure $ modifyAnns rdr $ addParens (isOperator $ unLoc rdr)
   let lie = reLocA $ L src $ IEName
-#if MIN_VERSION_ghc(9,5,0)
                                noExtField
-#endif
                                rdr
       x = reLocA $ L top $ IEVar
 #if MIN_VERSION_ghc(9,7,0)
@@ -577,11 +564,7 @@ extendHiding symbol (L l idecls) mlies df = do
 #endif
   x <- pure $ if hasSibling then first addComma x else x
   lies <- pure $ over _head (`setEntryDP` SameLine 1) lies
-#if MIN_VERSION_ghc(9,5,0)
   return $ L l idecls{ideclImportList = Just (EverythingBut, L l' $ x : lies)}
-#else
-  return $ L l idecls{ideclHiding = Just (True, L l' $ x : lies)}
-#endif
  where
   isOperator = not . all isAlphaNum . occNameString . rdrNameOcc
 
@@ -596,11 +579,7 @@ deleteFromImport (T.pack -> symbol) (L l idecl) (L lieLoc lies) _ = do
       lidecl' =
         L l $
           idecl
-#if MIN_VERSION_ghc(9,5,0)
             { ideclImportList = Just (Exactly, edited) }
-#else
-            { ideclHiding = Just (False, edited) }
-#endif
   pure lidecl'
  where
   deletedLies =
