@@ -10,8 +10,7 @@ import           Data.Functor                       (void)
 import qualified Data.List                          as T
 import           Data.Map.Strict                    as Map hiding (map)
 import           Data.String                        (fromString)
-import           Data.Text                          hiding (length, map,
-                                                     unlines)
+import           Data.Text                          (Text)
 import qualified Data.Text                          as Text
 import qualified Data.Text.Utf16.Rope.Mixed         as Rope
 import           Data.Version                       (Version (..))
@@ -225,7 +224,7 @@ semanticTokensFullDeltaTests =
 
 semanticTokensTests :: TestTree
 semanticTokensTests =
-  testGroup "other semantic Token test" $
+  testGroup "other semantic Token test"
     [ testCase "module import test" $ do
         let file1 = "TModuleA.hs"
         let file2 = "TModuleB.hs"
@@ -263,10 +262,9 @@ semanticTokensTests =
       goldenWithSemanticTokensWithDefaultConfig "pattern bind" "TPatternSynonym",
       goldenWithSemanticTokensWithDefaultConfig "type family" "TTypefamily",
       goldenWithSemanticTokensWithDefaultConfig "TUnicodeSyntax" "TUnicodeSyntax",
-      goldenWithSemanticTokensWithDefaultConfig "TQualifiedName" "TQualifiedName"
+      goldenWithSemanticTokensWithDefaultConfig "TQualifiedName" "TQualifiedName",
+      goldenWithSemanticTokensWithDefaultConfig "TDoc" "TDoc"
     ]
-      -- not supported in ghc92
-      ++ [goldenWithSemanticTokensWithDefaultConfig "TDoc" "TDoc" | ghcVersion > GHC92]
 
 semanticTokensDataTypeTests :: TestTree
 semanticTokensDataTypeTests =
