@@ -12,8 +12,8 @@ main :: IO ()
 main = defaultTestRunner $ testGroup "haskell-language-server"
     [ Config.tests
     , ConfigSchema.tests
-    , ignoreInEnv [HostOS Windows, GhcVer GHC92] "Tests gets stuck in ci" $ Format.tests
+    , ignoreInEnv [HostOS Windows] "Tests gets stuck in ci" Format.tests
     , FunctionalBadProject.tests
     , HieBios.tests
-    , ignoreInEnv [HostOS Windows] "Tests gets stuck in ci" $ Progress.tests
+    , ignoreInEnv [HostOS Windows] "Tests gets stuck in ci" Progress.tests
     ]
